@@ -50,14 +50,14 @@ or suffix were found:
 
 import logging
 import os
-import re
+import urllib.parse
 from functools import wraps
 from typing import FrozenSet, List, NamedTuple, Optional, Sequence, Union
 
 import idna
 
 from .cache import DiskCache, get_cache_dir
-from .remote import IP_RE, SCHEME_RE, looks_like_ip
+from .remote import IP_RE, lenient_netloc, looks_like_ip
 from .suffix_list import get_suffix_lists
 
 LOG = logging.getLogger("tldextract")
@@ -70,8 +70,6 @@ PUBLIC_SUFFIX_LIST_URLS = (
     "https://raw.githubusercontent.com/publicsuffix/list/master/public_suffix_list.dat",
 )
 
-_UNICODE_DOTS_RE = re.compile("[\u002e\u3002\uff0e\uff61]")
-
 
 class ExtractResult(NamedTuple):
     """namedtuple of a URL's subdomain, domain, and suffix."""
@@ -207,34 +205,58 @@ class TLDExtract:
 
     def __call__(
         self, url: str, include_psl_private_domains: Optional[bool] = None
+    ) -> ExtractResult:
+        """Alias for `extract_str`."""
+        return self.extract_str(url, include_psl_private_domains)
+
+    def extract_str(
+        self, url: str, include_psl_private_domains: Optional[bool] = None
     ) -> ExtractResult:
         """
         Takes a string URL and splits it into its subdomain, domain, and
-        suffix (effective TLD, gTLD, ccTLD, etc.) component.
+        suffix (effective TLD, gTLD, ccTLD, etc.) components.
 
-        >>> extract = TLDExtract()
-        >>> extract('http://forums.news.cnn.com/')
+        >>> extractor = TLDExtract()
+        >>> extractor.extract_str('http://forums.news.cnn.com/')
         ExtractResult(subdomain='forums.news', domain='cnn', suffix='com')
-        >>> extract('http://forums.bbc.co.uk/')
+        >>> extractor.extract_str('http://forums.bbc.co.uk/')
         ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk')
         """
+        return self._extract_netloc(lenient_netloc(url), include_psl_private_domains)
 
-        netloc = (
-            SCHEME_RE.sub("", url)
-            .partition("/")[0]
-            .partition("?")[0]
-            .partition("#")[0]
-            .split("@")[-1]
-            .partition(":")[0]
-            .strip()
-            .rstrip(".")
+    def extract_urllib(
+        self,
+        url: Union[urllib.parse.ParseResult, urllib.parse.SplitResult],
+        include_psl_private_domains: Optional[bool] = None,
+    ) -> ExtractResult:
+        """
+        Takes the output of urllib.parse URL parsing methods and further splits
+        the parsed URL into its subdomain, domain, and suffix (effective TLD,
+        gTLD, ccTLD, etc.) components.
+
+        This method is like `extract_str` but faster, as the string's domain
+        name has already been parsed.
+
+        >>> extractor = TLDExtract()
+        >>> extractor.extract_urllib(urllib.parse.urlsplit('http://forums.news.cnn.com/'))
+        ExtractResult(subdomain='forums.news', domain='cnn', suffix='com')
+        >>> extractor.extract_urllib(urllib.parse.urlsplit('http://forums.bbc.co.uk/'))
+        ExtractResult(subdomain='forums', domain='bbc', suffix='co.uk')
+        """
+        return self._extract_netloc(url.netloc, include_psl_private_domains)
+
+    def _extract_netloc(
+        self, netloc: str, include_psl_private_domains: Optional[bool]
+    ) -> ExtractResult:
+        labels = (
+            netloc.replace("\u3002", "\u002e")
+            .replace("\uff0e", "\u002e")
+            .replace("\uff61", "\u002e")
+            .split(".")
         )
 
-        labels = _UNICODE_DOTS_RE.split(netloc)
-
-        translations = [_decode_punycode(label) for label in labels]
         suffix_index = self._get_tld_extractor().suffix_index(
-            translations, include_psl_private_domains=include_psl_private_domains
+            labels, include_psl_private_domains=include_psl_private_domains
         )
 
         suffix = ".".join(labels[suffix_index:])
@@ -328,6 +350,28 @@ class _PublicSuffixListTLDExtractor:
         self.tlds_incl_private = frozenset(public_tlds + private_tlds + extra_tlds)
         self.tlds_excl_private = frozenset(public_tlds + extra_tlds)
 
+        public_false_tlds = self.get_false_intermediate_suffixes(public_tlds)
+        private_false_tlds = self.get_false_intermediate_suffixes(private_tlds)
+        extra_false_tlds = self.get_false_intermediate_suffixes(extra_tlds)
+        self.false_tlds_incl_private = frozenset(
+            public_false_tlds + private_false_tlds + extra_false_tlds
+        )
+        self.false_tlds_excl_private = frozenset(public_false_tlds + extra_false_tlds)
+
+    def get_false_intermediate_suffixes(self, tlds: List[str]) -> List[str]:
+        """From list of suffixes, identify false intermediate suffixes.
+
+        Example: If valid TLDs include only ["a.b.c.d", "d"], then
+        ["b.c.d", "c.d"] are false intermediate suffixes.
+        """
+        valid_tlds = set(tlds)
+        false_tlds = set()
+        for tld in valid_tlds:
+            labels = tld.split(".")
+            variants = {".".join(labels[-i:]) for i in range(1, len(labels))}
+            false_tlds.update(variants)
+        return list(false_tlds.difference(valid_tlds))
+
     def tlds(
         self, include_psl_private_domains: Optional[bool] = None
     ) -> FrozenSet[str]:
@@ -341,28 +385,54 @@ class _PublicSuffixListTLDExtractor:
             else self.tlds_excl_private
         )
 
+    def false_tlds(
+        self, include_psl_private_domains: Optional[bool] = None
+    ) -> FrozenSet[str]:
+        """Get the currently filtered list of false intermediate suffixes."""
+        if include_psl_private_domains is None:
+            include_psl_private_domains = self.include_psl_private_domains
+
+        return (
+            self.false_tlds_incl_private
+            if include_psl_private_domains
+            else self.false_tlds_excl_private
+        )
+
     def suffix_index(
-        self, lower_spl: List[str], include_psl_private_domains: Optional[bool] = None
+        self, spl: List[str], include_psl_private_domains: Optional[bool] = None
     ) -> int:
         """Returns the index of the first suffix label.
         Returns len(spl) if no suffix is found
         """
         tlds = self.tlds(include_psl_private_domains)
-        length = len(lower_spl)
-        for i in range(length):
-            maybe_tld = ".".join(lower_spl[i:])
-            exception_tld = "!" + maybe_tld
-            if exception_tld in tlds:
-                return i + 1
+        false_tlds = self.false_tlds(include_psl_private_domains)
+        i = len(spl)
+        j = i
+        maybe_tld = ""
+        prev_maybe_tld = ""
+        for label in reversed(spl):
+            maybe_tld = (
+                f"{_decode_punycode(label)}.{maybe_tld}"
+                if maybe_tld
+                else _decode_punycode(label)
+            )
+
+            if "!" + maybe_tld in tlds:
+                return j
+            if "*." + prev_maybe_tld in tlds:
+                return j - 1
 
             if maybe_tld in tlds:
-                return i
-
-            wildcard_tld = "*." + ".".join(lower_spl[i + 1 :])
-            if wildcard_tld in tlds:
-                return i
-
-        return length
+                j -= 1
+                i = j
+                prev_maybe_tld = maybe_tld
+                continue
+            if maybe_tld in false_tlds:
+                j -= 1
+                prev_maybe_tld = maybe_tld
+                continue
+            break
+        return i
 
 
 def _decode_punycode(label: str) -> str:
